@@ -286,8 +286,9 @@ class TVariousFrames(TestCase):
         ],
         [
             'LINK', b'TIT1http://www.example.org/TIT1.txt\x00',
-            ("TIT1", 'http://www.example.org/TIT1.txt'), '',
-            dict(frameid='TIT1', url='http://www.example.org/TIT1.txt')
+            ("TIT1", 'http://www.example.org/TIT1.txt', b''), '',
+            dict(frameid='TIT1', url='http://www.example.org/TIT1.txt',
+                 data=b'')
         ],
         [
             'LINK', b'COMMhttp://www.example.org/COMM.txt\x00engfoo',
@@ -442,13 +443,18 @@ class TVariousFrames(TestCase):
         ],
         [
             'LNK', b'TT1http://www.example.org/TIT1.txt\x00',
-            ("TT1", 'http://www.example.org/TIT1.txt'), '',
-            dict(frameid='TT1', url='http://www.example.org/TIT1.txt')
+            ("TT1", 'http://www.example.org/TIT1.txt', b''), '',
+            dict(frameid='TT1', url='http://www.example.org/TIT1.txt',
+                 data=b'')
         ],
         [
             'CRM', b'foo@example.org\x00test\x00woo', b'woo', '',
             dict(owner='foo@example.org', desc='test', data=b'woo')
         ],
+        [
+            'CRM', b'\x00\x00', b'', '',
+            dict(owner='', desc='', data=b'')
+        ],
     ]
 
     def _get_frame(self, id_):
@@ -571,10 +577,6 @@ class TCRA(TestCase):
         self.assertEqual(new.preview_length, 2)
         self.assertEqual(new.data, b"foo")
 
-        frame = CRA(owner="a", preview_start=1, preview_length=2)
-        new = AENC(frame)
-        self.assertFalse(hasattr(new, "data"))
-
 
 class TPIC(TestCase):
 
@@ -824,7 +826,7 @@ class TLINK(TestCase):
         frame._pprint()
 
         frame = LINK(frameid="TPE1", url="http://foo.bar")
-        self.assertEqual(frame.HashKey, "LINK:TPE1:http://foo.bar")
+        self.assertEqual(frame.HashKey, "LINK:TPE1:http://foo.bar:")
 
 
 class TAENC(TestCase):
@@ -1199,6 +1201,8 @@ class TTPE1(TestCase):
     def test_junkframe(self):
         self.assertRaises(
             ID3JunkFrameError, TPE1._fromData, _24, 0, b"")
+        self.assertRaises(
+            ID3JunkFrameError, TPE1._fromData, _24, 0, b'\x03A\xff\xfe')
 
     def test_lengthone_utf16(self):
         tpe1 = TPE1._fromData(_24, 0, b'\x01\x00')
@@ -1212,6 +1216,24 @@ class TTPE1(TestCase):
             _24, 0, b'\x01\xff\xfeH\x00e\x00l\x00l\x00o\x00\x00')
         self.assertEquals(tpe1, [u'Hello'])
 
+        tpe1 = TPE1._fromData(
+            _24, 0, b'\x02\x00H\x00e\x00l\x00l\x00o\x00')
+        self.assertEquals(tpe1, [u'Hello'])
+
+    def test_utf_16_missing_bom(self):
+        tpe1 = TPE1._fromData(
+            _24, 0, b'\x01H\x00e\x00l\x00l\x00o\x00\x00\x00')
+        self.assertEquals(tpe1, [u'Hello'])
+
+    def test_utf_16_missing_bom_wrong_nullterm(self):
+        tpe1 = TPE1._fromData(
+            _24, 0, b'\x01H\x00e\x00l\x00l\x00o\x00\x00')
+        self.assertEquals(tpe1, [u'Hello'])
+
+        tpe1 = TPE1._fromData(
+            _24, 0, b'\x01f\x00o\x00o\x00\x00\x00b\x00a\x00r\x00\x00')
+        self.assertEquals(tpe1, [u"foo", u"bar"])
+
     def test_zlib_bpi(self):
         tpe1 = TPE1(encoding=0, text="a" * (0xFFFF - 2))
         data = save_frame(tpe1)
