@@ -1,13 +1,17 @@
 # -*- coding: utf-8 -*-
 
-from mutagen._util import DictMixin, cdata, insert_bytes, delete_bytes
-from mutagen._util import decode_terminated, dict_match, enum, get_size
-from mutagen._util import BitReader, BitReaderError, resize_bytes
+from mutagen._util import DictMixin, cdata, insert_bytes, delete_bytes, \
+    decode_terminated, dict_match, enum, get_size, BitReader, BitReaderError, \
+    resize_bytes, seek_end, mmap_move, verify_fileobj, fileobj_name, \
+    read_full, flags, resize_file, fallback_move
 from mutagen._compat import text_type, itervalues, iterkeys, iteritems, PY2, \
-    cBytesIO, xrange
-from tests import TestCase
+    cBytesIO, xrange, BytesIO
+from tests import TestCase, get_temp_empty
+import os
 import random
+import tempfile
 import mmap
+import errno
 
 try:
     import fcntl
@@ -261,9 +265,129 @@ class Tcdata(TestCase):
         self.failIf(cdata.test_bit(v, 13))
 
 
+class Tresize_file(TestCase):
+
+    def get_named_file(self, content):
+        filename = get_temp_empty()
+        h = open(filename, "wb+")
+        h.write(content)
+        h.seek(0)
+        return h
+
+    def test_resize(self):
+        with self.get_named_file(b"") as h:
+            resize_file(h, 0)
+            self.assertEqual(os.path.getsize(h.name), 0)
+            self.assertRaises(ValueError, resize_file, h, -1)
+            resize_file(h, 1)
+            self.assertEqual(os.path.getsize(h.name), 1)
+            h.seek(0)
+            self.assertEqual(h.read(), b"\x00")
+            resize_file(h, 2 ** 17)
+            self.assertEqual(os.path.getsize(h.name), 2 ** 17 + 1)
+            h.seek(0)
+            self.assertEqual(h.read(), b"\x00" * (2 ** 17 + 1))
+
+    def test_resize_content(self):
+        with self.get_named_file(b"abc") as h:
+            self.assertRaises(ValueError, resize_file, h, -4)
+            resize_file(h, -1)
+            h.seek(0)
+            self.assertEqual(h.read(), b"ab")
+            resize_file(h, 2)
+            h.seek(0)
+            self.assertEqual(h.read(), b"ab\x00\x00")
+
+    def test_resize_dev_full(self):
+
+        def raise_no_space(*args):
+            raise IOError(errno.ENOSPC, os.strerror(errno.ENOSPC))
+
+        # fail on first write
+        h = BytesIO(b"abc")
+        h.write = raise_no_space
+        self.assertRaises(IOError, resize_file, h, 1)
+        h.seek(0, 2)
+        self.assertEqual(h.tell(), 3)
+
+        # fail on flush
+        h = BytesIO(b"abc")
+        h.flush = raise_no_space
+        self.assertRaises(IOError, resize_file, h, 1)
+        h.seek(0, 2)
+        self.assertEqual(h.tell(), 3)
+
+
+class TMoveMixin(object):
+
+    MOVE = None
+
+    def file(self, contents):
+        temp = tempfile.TemporaryFile()
+        temp.write(contents)
+        temp.flush()
+        temp.seek(0)
+        return temp
+
+    def read(self, fobj):
+        fobj.seek(0, 0)
+        return fobj.read()
+
+    def test_basic(self):
+        with self.file(b"abc123") as h:
+            self.MOVE(h, 0, 1, 4)
+            self.assertEqual(self.read(h), b"bc1223")
+
+        with self.file(b"abc123") as h:
+            self.MOVE(h, 1, 0, 4)
+            self.assertEqual(self.read(h), b"aabc13")
+
+    def test_invalid_params(self):
+        with self.file(b"foo") as o:
+            self.assertRaises(ValueError, self.MOVE, o, -1, 0, 0)
+            self.assertRaises(ValueError, self.MOVE, o, 0, -1, 0)
+            self.assertRaises(ValueError, self.MOVE, o, 0, 0, -1)
+
+    def test_outside_file(self):
+        with self.file(b"foo") as o:
+            self.assertRaises(ValueError, self.MOVE, o, 0, 0, 4)
+            self.assertRaises(ValueError, self.MOVE, o, 0, 1, 3)
+            self.assertRaises(ValueError, self.MOVE, o, 1, 0, 3)
+
+    def test_ok(self):
+        with self.file(b"foo") as o:
+            self.MOVE(o, 0, 1, 2)
+            self.MOVE(o, 1, 0, 2)
+
+    def test_larger_than_page_size(self):
+        off = mmap.ALLOCATIONGRANULARITY
+        with self.file(b"f" * off * 2) as o:
+            self.MOVE(o, off, off + 1, off - 1)
+            self.MOVE(o, off + 1, off, off - 1)
+
+        with self.file(b"f" * off * 2 + b"x") as o:
+            self.MOVE(o, off * 2 - 1, off * 2, 1)
+            self.assertEqual(self.read(o)[-3:], b"fxx")
+
+
+class Tfallback_move(TestCase, TMoveMixin):
+
+    MOVE = staticmethod(fallback_move)
+
+
+class MmapMove(TestCase, TMoveMixin):
+
+    MOVE = staticmethod(mmap_move)
+
+    def test_stringio(self):
+        self.assertRaises(mmap.error, mmap_move, cBytesIO(), 0, 0, 0)
+
+    def test_no_fileno(self):
+        self.assertRaises(mmap.error, mmap_move, object(), 0, 0, 0)
+
+
 class FileHandling(TestCase):
     def file(self, contents):
-        import tempfile
         temp = tempfile.TemporaryFile()
         temp.write(contents)
         temp.flush()
@@ -275,103 +399,109 @@ class FileHandling(TestCase):
         return fobj.read()
 
     def test_resize_decrease(self):
-        o = self.file(b'abcd')
-        resize_bytes(o, 2, 1, 1)
-        self.assertEqual(self.read(o), b"abd")
+        with self.file(b'abcd') as o:
+            resize_bytes(o, 2, 1, 1)
+            self.assertEqual(self.read(o), b"abd")
 
     def test_resize_increase(self):
-        o = self.file(b'abcd')
-        resize_bytes(o, 2, 4, 1)
-        self.assertEqual(self.read(o), b"abcd\x00d")
+        with self.file(b'abcd') as o:
+            resize_bytes(o, 2, 4, 1)
+            self.assertEqual(self.read(o), b"abcd\x00d")
 
     def test_resize_nothing(self):
-        o = self.file(b'abcd')
-        resize_bytes(o, 2, 2, 1)
-        self.assertEqual(self.read(o), b"abcd")
+        with self.file(b'abcd') as o:
+            resize_bytes(o, 2, 2, 1)
+            self.assertEqual(self.read(o), b"abcd")
 
     def test_insert_into_empty(self):
-        o = self.file(b'')
-        insert_bytes(o, 8, 0)
-        self.assertEquals(b'\x00' * 8, self.read(o))
+        with self.file(b'') as o:
+            insert_bytes(o, 8, 0)
+            self.assertEqual(b'\x00' * 8, self.read(o))
 
     def test_insert_before_one(self):
-        o = self.file(b'a')
-        insert_bytes(o, 8, 0)
-        self.assertEquals(b'a' + b'\x00' * 7 + b'a', self.read(o))
+        with self.file(b'a') as o:
+            insert_bytes(o, 8, 0)
+            self.assertEqual(b'a' + b'\x00' * 7 + b'a', self.read(o))
 
     def test_insert_after_one(self):
-        o = self.file(b'a')
-        insert_bytes(o, 8, 1)
-        self.assertEquals(b'a' + b'\x00' * 8, self.read(o))
+        with self.file(b'a') as o:
+            insert_bytes(o, 8, 1)
+            self.assertEqual(b'a' + b'\x00' * 8, self.read(o))
+
+    def test_insert_after_file(self):
+        with self.file(b'a') as o:
+            self.assertRaises(ValueError, insert_bytes, o, 1, 2)
 
     def test_smaller_than_file_middle(self):
-        o = self.file(b'abcdefghij')
-        insert_bytes(o, 4, 4)
-        self.assertEquals(b'abcdefghefghij', self.read(o))
+        with self.file(b'abcdefghij') as o:
+            insert_bytes(o, 4, 4)
+            self.assertEqual(b'abcdefghefghij', self.read(o))
 
     def test_smaller_than_file_to_end(self):
-        o = self.file(b'abcdefghij')
-        insert_bytes(o, 4, 6)
-        self.assertEquals(b'abcdefghijghij', self.read(o))
+        with self.file(b'abcdefghij') as o:
+            insert_bytes(o, 4, 6)
+            self.assertEqual(b'abcdefghijghij', self.read(o))
 
     def test_smaller_than_file_across_end(self):
-        o = self.file(b'abcdefghij')
-        insert_bytes(o, 4, 8)
-        self.assertEquals(b'abcdefghij\x00\x00ij', self.read(o))
+        with self.file(b'abcdefghij') as o:
+            insert_bytes(o, 4, 8)
+            self.assertEqual(b'abcdefghij\x00\x00ij', self.read(o))
 
     def test_smaller_than_file_at_end(self):
-        o = self.file(b'abcdefghij')
-        insert_bytes(o, 3, 10)
-        self.assertEquals(b'abcdefghij\x00\x00\x00', self.read(o))
+        with self.file(b'abcdefghij') as o:
+            insert_bytes(o, 3, 10)
+            self.assertEqual(b'abcdefghij\x00\x00\x00', self.read(o))
 
     def test_smaller_than_file_at_beginning(self):
-        o = self.file(b'abcdefghij')
-        insert_bytes(o, 3, 0)
-        self.assertEquals(b'abcabcdefghij', self.read(o))
+        with self.file(b'abcdefghij') as o:
+            insert_bytes(o, 3, 0)
+            self.assertEqual(b'abcabcdefghij', self.read(o))
 
     def test_zero(self):
-        o = self.file(b'abcdefghij')
-        self.assertRaises((AssertionError, ValueError), insert_bytes, o, 0, 1)
+        with self.file(b'abcdefghij') as o:
+            insert_bytes(o, 0, 1)
+            self.assertEqual(b'abcdefghij', self.read(o))
 
     def test_negative(self):
-        o = self.file(b'abcdefghij')
-        self.assertRaises((AssertionError, ValueError), insert_bytes, o, 8, -1)
+        with self.file(b'abcdefghij') as o:
+            self.assertRaises(ValueError, insert_bytes, o, 8, -1)
 
     def test_delete_one(self):
-        o = self.file(b'a')
-        delete_bytes(o, 1, 0)
-        self.assertEquals(b'', self.read(o))
+        with self.file(b'a') as o:
+            delete_bytes(o, 1, 0)
+            self.assertEqual(b'', self.read(o))
 
     def test_delete_first_of_two(self):
-        o = self.file(b'ab')
-        delete_bytes(o, 1, 0)
-        self.assertEquals(b'b', self.read(o))
+        with self.file(b'ab') as o:
+            delete_bytes(o, 1, 0)
+            self.assertEqual(b'b', self.read(o))
 
     def test_delete_second_of_two(self):
-        o = self.file(b'ab')
-        delete_bytes(o, 1, 1)
-        self.assertEquals(b'a', self.read(o))
+        with self.file(b'ab') as o:
+            delete_bytes(o, 1, 1)
+            self.assertEqual(b'a', self.read(o))
 
     def test_delete_third_of_two(self):
-        o = self.file(b'ab')
-        self.assertRaises(AssertionError, delete_bytes, o, 1, 2)
+        with self.file(b'ab') as o:
+            self.assertRaises(ValueError, delete_bytes, o, 1, 2)
 
     def test_delete_middle(self):
-        o = self.file(b'abcdefg')
-        delete_bytes(o, 3, 2)
-        self.assertEquals(b'abfg', self.read(o))
+        with self.file(b'abcdefg') as o:
+            delete_bytes(o, 3, 2)
+            self.assertEqual(b'abfg', self.read(o))
 
     def test_delete_across_end(self):
-        o = self.file(b'abcdefg')
-        self.assertRaises(AssertionError, delete_bytes, o, 4, 8)
+        with self.file(b'abcdefg') as o:
+            self.assertRaises(ValueError, delete_bytes, o, 4, 8)
 
     def test_delete_zero(self):
-        o = self.file(b'abcdefg')
-        self.assertRaises(AssertionError, delete_bytes, o, 0, 3)
+        with self.file(b'abcdefg') as o:
+            delete_bytes(o, 0, 3)
+            self.assertEqual(b'abcdefg', self.read(o))
 
     def test_delete_negative(self):
-        o = self.file(b'abcdefg')
-        self.assertRaises(AssertionError, delete_bytes, o, 4, -8)
+        with self.file(b'abcdefg') as o:
+            self.assertRaises(ValueError, delete_bytes, o, 4, -8)
 
     def test_insert_6106_79_51760(self):
         # This appears to be due to ANSI C limitations in read/write on rb+
@@ -379,9 +509,9 @@ class FileHandling(TestCase):
         # code for transfers of this or similar sizes.
         data = u''.join(map(text_type, xrange(12574)))  # 51760 bytes
         data = data.encode("ascii")
-        o = self.file(data)
-        insert_bytes(o, 6106, 79)
-        self.failUnless(data[:6106 + 79] + data[79:] == self.read(o))
+        with self.file(data) as o:
+            insert_bytes(o, 6106, 79)
+            self.failUnless(data[:6106 + 79] + data[79:] == self.read(o))
 
     def test_delete_6106_79_51760(self):
         # This appears to be due to ANSI C limitations in read/write on rb+
@@ -389,9 +519,9 @@ class FileHandling(TestCase):
         # code for transfers of this or similar sizes.
         data = u''.join(map(text_type, xrange(12574)))  # 51760 bytes
         data = data.encode("ascii")
-        o = self.file(data[:6106 + 79] + data[79:])
-        delete_bytes(o, 6106, 79)
-        self.failUnless(data == self.read(o))
+        with self.file(data[:6106 + 79] + data[79:]) as o:
+            delete_bytes(o, 6106, 79)
+            self.failUnless(data == self.read(o))
 
     # Generate a bunch of random insertions, apply them, delete them,
     # and make sure everything is still correct.
@@ -410,62 +540,42 @@ class FileHandling(TestCase):
                         "Given testing parameters make this test useless")
         for j in xrange(num_runs):
             data = b"ABCDEFGHIJKLMNOPQRSTUVWXYZ" * 1024
-            fobj = self.file(data)
-            filesize = len(data)
-            # Generate the list of changes to apply
-            changes = []
-            for i in xrange(num_changes):
-                change_size = random.randrange(
-                    min_change_size, max_change_size)
-                change_offset = random.randrange(0, filesize)
-                filesize += change_size
-                changes.append((change_offset, change_size))
+            with self.file(data) as fobj:
+                filesize = len(data)
+                # Generate the list of changes to apply
+                changes = []
+                for i in xrange(num_changes):
+                    change_size = random.randrange(
+                        min_change_size, max_change_size)
+                    change_offset = random.randrange(0, filesize)
+                    filesize += change_size
+                    changes.append((change_offset, change_size))
 
-            # Apply the changes, and make sure they all took.
-            for offset, size in changes:
-                buffer_size = random.randrange(
-                    min_buffer_size, max_buffer_size)
-                insert_bytes(fobj, size, offset, BUFFER_SIZE=buffer_size)
-            fobj.seek(0)
-            self.failIfEqual(fobj.read(len(data)), data)
-            fobj.seek(0, 2)
-            self.failUnlessEqual(fobj.tell(), filesize)
+                # Apply the changes, and make sure they all took.
+                for offset, size in changes:
+                    buffer_size = random.randrange(
+                        min_buffer_size, max_buffer_size)
+                    insert_bytes(fobj, size, offset, BUFFER_SIZE=buffer_size)
+                fobj.seek(0)
+                self.failIfEqual(fobj.read(len(data)), data)
+                fobj.seek(0, 2)
+                self.failUnlessEqual(fobj.tell(), filesize)
 
-            # Then, undo them.
-            changes.reverse()
-            for offset, size in changes:
-                buffer_size = random.randrange(
-                    min_buffer_size, max_buffer_size)
-                delete_bytes(fobj, size, offset, BUFFER_SIZE=buffer_size)
-            fobj.seek(0)
-            self.failUnless(fobj.read() == data)
-
-
-class FileHandlingMockedMMapMove(FileHandling):
-
-    def setUp(self):
-        class MockMMap(object):
-            def __init__(self, *args, **kwargs):
-                pass
-
-            def move(self, dest, src, count):
-                raise ValueError
-
-            def close(self):
-                pass
-
-        self._orig_mmap = mmap.mmap
-        mmap.mmap = MockMMap
-
-    def tearDown(self):
-        mmap.mmap = self._orig_mmap
+                # Then, undo them.
+                changes.reverse()
+                for offset, size in changes:
+                    buffer_size = random.randrange(
+                        min_buffer_size, max_buffer_size)
+                    delete_bytes(fobj, size, offset, BUFFER_SIZE=buffer_size)
+                fobj.seek(0)
+                self.failUnless(fobj.read() == data)
 
 
 class FileHandlingMockedMMap(FileHandling):
 
     def setUp(self):
         def MockMMap2(*args, **kwargs):
-            raise EnvironmentError
+            raise mmap.error
 
         self._orig_mmap = mmap.mmap
         mmap.mmap = MockMMap2
@@ -512,6 +622,94 @@ class Tenum(TestCase):
         self.assertTrue(isinstance(repr(Foo.FOO), str))
 
 
+class Tflags(TestCase):
+
+    def test_enum(self):
+        @flags
+        class Foo(object):
+            FOO = 1
+            BAR = 2
+
+        self.assertEqual(Foo.FOO, 1)
+        self.assertTrue(isinstance(Foo.FOO, Foo))
+        self.assertEqual(repr(Foo.FOO), "<Foo.FOO: 1>")
+        self.assertEqual(repr(Foo(3)), "<Foo.FOO | Foo.BAR: 3>")
+        self.assertEqual(repr(Foo(42)), "<Foo.BAR | 40: 42>")
+        self.assertEqual(str(Foo(42)), "Foo.BAR | 40")
+        self.assertEqual(int(Foo(42)), 42)
+        self.assertEqual(str(Foo(1)), "Foo.FOO")
+        self.assertEqual(int(Foo(1)), 1)
+        self.assertEqual(str(Foo(0)), "0")
+
+        self.assertTrue(isinstance(str(Foo.FOO), str))
+        self.assertTrue(isinstance(repr(Foo.FOO), str))
+
+
+class Tverify_fileobj(TestCase):
+
+    def test_verify_fileobj_fail(self):
+        self.assertRaises(ValueError, verify_fileobj, object())
+        with tempfile.TemporaryFile(mode="rb") as h:
+            self.assertRaises(ValueError, verify_fileobj, h, writable=True)
+
+    def test_verify_fileobj(self):
+        with tempfile.TemporaryFile(mode="rb") as h:
+            verify_fileobj(h)
+
+        with tempfile.TemporaryFile(mode="rb+") as h:
+            verify_fileobj(h, writable=True)
+
+
+class Tfileobj_name(TestCase):
+
+    def test_fileobj_name_other_type(self):
+
+        class Foo(object):
+            name = 123
+
+        self.assertEqual(fileobj_name(Foo()), "123")
+
+    def test_fileobj_name(self):
+        with tempfile.TemporaryFile(mode="rb") as h:
+            self.assertEqual(fileobj_name(h), text_type(h.name))
+
+
+class Tseek_end(TestCase):
+
+    def file(self, contents):
+        temp = tempfile.TemporaryFile()
+        temp.write(contents)
+        temp.flush()
+        temp.seek(0)
+        return temp
+
+    def test_seek_end(self):
+        with self.file(b"foo") as f:
+            seek_end(f, 2)
+            self.assertEqual(f.tell(), 1)
+            seek_end(f, 3)
+            self.assertEqual(f.tell(), 0)
+            seek_end(f, 4)
+            self.assertEqual(f.tell(), 0)
+            seek_end(f, 0)
+            self.assertEqual(f.tell(), 3)
+            self.assertRaises(ValueError, seek_end, f, -1)
+
+    def test_seek_end_pos(self):
+        with self.file(b"foo") as f:
+            f.seek(10)
+            seek_end(f, 10)
+            self.assertEqual(f.tell(), 0)
+
+
+class Tread_full(TestCase):
+
+    def test_read_full(self):
+        fileobj = cBytesIO()
+        self.assertRaises(ValueError, read_full, fileobj, -3)
+        self.assertRaises(IOError, read_full, fileobj, 3)
+
+
 class Tget_size(TestCase):
 
     def test_get_size(self):
@@ -546,6 +744,8 @@ class Tdecode_terminated(TestCase):
             UnicodeDecodeError, decode_terminated, b"\xff\xfe\x00", "utf-16")
         # not null terminated
         self.assertRaises(ValueError, decode_terminated, b"abc", "utf-8")
+        self.assertRaises(
+            ValueError, decode_terminated, b"\xff\xfea\x00", "utf-16")
         # invalid encoding
         self.assertRaises(LookupError, decode_terminated, b"abc", "foobar")
 
@@ -573,6 +773,22 @@ class TBitReader(TestCase):
             v = r.bits(i) << (64 - i) | r.bits(64 - i)
             self.assertEqual(v, ref)
 
+    def test_bits_null(self):
+        r = BitReader(cBytesIO(b""))
+        self.assertEqual(r.bits(0), 0)
+
+    def test_bits_error(self):
+        r = BitReader(cBytesIO(b""))
+        self.assertRaises(ValueError, r.bits, -1)
+
+    def test_bytes_error(self):
+        r = BitReader(cBytesIO(b""))
+        self.assertRaises(ValueError, r.bytes, -1)
+
+    def test_skip_error(self):
+        r = BitReader(cBytesIO(b""))
+        self.assertRaises(ValueError, r.skip, -1)
+
     def test_read_too_much(self):
         r = BitReader(cBytesIO(b""))
         self.assertEqual(r.bits(0), 0)
